package actions;

import java.util.Hashtable;

import javax.servlet.http.HttpServletRequest;

/**
 * The Class Action.
 * 
 * This abstract class is used to map Action Names to Actions via a
 * hashtable for quick lookup.
 * 
 * Actions will follow this idiom:
 *
 * get Session.
 * begin Transaction
 * 
 * get errors from form (if applicable) or create errors ArrayList.
 * attach errors to request.
 * if errors return.
 *
 * do work.
 * return JSP forward page.
 *
 * commit Transaction
 * set request attributes
 *
 */
public abstract class Action {
	
	private static Hashtable<String, Action> hash = new Hashtable<String, Action>();
	
	/**
	 * Adds the specified Action to the hashtable for lookup
	 * 
	 * @param Action
	 */
	public static void add(Action a) {
		hash.put(a.getName(), a);
	}
	
	/**
	 * Redirects the specified Request to an Action according
	 * to the specified action Name.  Returns null if the Action
	 * Name is not found.
	 * 
	 * @param Action Name
	 * @param Servlet Request
	 * 
	 * @return JSP Page Name
	 */
	public static String perform(String name, HttpServletRequest request) {
		Action a = hash.get(name);
		if (a == null)
			return null;
		return a.perform(request);
	}
	
	/**
	 * Returns the name of the Action, used for inserting actions
	 * into the hastable for future lookup.
	 * 
	 * @return Action Name
	 */
	public abstract String getName();
	
	/**
	 * Performs the request according to the action
	 * 
	 * @param Servlet Request
	 * 
	 * @return JSP Page Name
	 */
	public abstract String perform(HttpServletRequest request);
}
