package actions;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.jasypt.util.password.BasicPasswordEncryptor;

import util.HibernateUtil;
import beans.User;
import daos.UserDAO;

import forms.LoginForm;

public class LoginAction extends Action {

	private UserDAO userDAO;
	private BasicPasswordEncryptor passwordEncryptor;

	public LoginAction() {
		userDAO = new UserDAO();
		passwordEncryptor = new BasicPasswordEncryptor();
	}

	/* (non-Javadoc)
	 * @see actions.Action#getName()
	 */
	@Override
	public String getName() {
		return "login";
	}

	/* (non-Javadoc)
	 * @see actions.Action#perform(javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public String perform(HttpServletRequest request) {
		HttpSession session = request.getSession();
        HibernateUtil.beginTransaction(); 
        
		// Set form as a request attribute so we can provide any values back to
		// the user in case of errors	
        LoginForm loginForm = new LoginForm(request);
		request.setAttribute("form", loginForm);
		
		// Any validation errors, return to the form and ask again.
		// Note, we set up error list (used to store any errors we might
		// encounter below)
		List<String> errors = loginForm.getErrors();
		request.setAttribute("errors", errors);
		if (!errors.isEmpty()) {
			HibernateUtil.commitTransaction();
			return "index.jsp";
		}

		String name = loginForm.getName();
		String pass = loginForm.getPass();
			
		User user = userDAO.getUserByName(name);
		
		if (user == null) {
			errors.add("invalid user name or password");
			HibernateUtil.commitTransaction();
			return "index.jsp";
		}
		
		if (!passwordEncryptor.checkPassword(pass, user.getPass())) {
			errors.add("invalid user name or password");
			HibernateUtil.commitTransaction();
			return "index.jsp";
		} 
		
		HibernateUtil.commitTransaction();
		
		//yay you win
		//see the difference between putting this before & after the commit
		session.setAttribute("userId", user.getId());

		return Action.perform("sites", request);
	}
	
}