package actions;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.jasypt.util.password.BasicPasswordEncryptor;

import beans.User;
import daos.UserDAO;
import forms.RegisterForm;
import util.HibernateUtil;
import net.sf.javainetlocator.*;

public class RegisterAction extends Action {

	private UserDAO userDAO;
	private BasicPasswordEncryptor passwordEncryptor;

	public RegisterAction() {
		userDAO = new UserDAO();
		passwordEncryptor = new BasicPasswordEncryptor();
	}

	/* (non-Javadoc)
	 * @see actions.Action#getName()
	 */
	@Override
	public String getName() {
		return "register";
	}

	/* (non-Javadoc)
	 * @see actions.Action#perform(javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public String perform(HttpServletRequest request) {
		HttpSession session = request.getSession();
		HibernateUtil.beginTransaction();
		
		// Set form as a request attribute so we can provide any values back to
		// the user in case of errors
		RegisterForm registerForm = new RegisterForm(request);
        request.setAttribute("form", registerForm);

		// Any validation errors, return to the form and ask again.
		// Note, we set up error list (used to store any errors we might
		// encounter below)
		List<String> errors = registerForm.getErrors();
		request.setAttribute("errors", errors);
		if (!errors.isEmpty()) {
			HibernateUtil.commitTransaction();
			return "register.jsp";
		}

		// check and see if user name is already taken
		String name = registerForm.getName();
		
		if (!userDAO.isUserNameAvailable(name)) {
			errors.add("sorry, that user name is taken.");
			HibernateUtil.commitTransaction();
			return "register.jsp";
		}
		
		String encryptedPass = passwordEncryptor.encryptPassword(registerForm.getPass());
		String lastPage = "sites";
		String ip = request.getRemoteAddr();
		String geoLoc = "";
		try {
			//this geoLoc is intended for the City or something like that...
			geoLoc = (InetAddressLocator.getLocale(ip.getBytes())).getCountry();
		} catch(InetAddressLocatorException e) {}

		User user = new User(name, encryptedPass, lastPage, ip, geoLoc);
		
		userDAO.makePersistent(user);
		HibernateUtil.commitTransaction();
		session.setAttribute("userId", user.getId());
		session.setAttribute("user", user);
		
		return "sites.jsp";
	}

}
