package actions.manage;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.jasypt.util.password.BasicPasswordEncryptor;

import actions.Action;
import beans.User;
import daos.UserDAO;
import forms.ChangeForm;
import util.HibernateUtil;
import net.sf.javainetlocator.*;

public class ChangeAction extends Action {

	private UserDAO userDAO;
	private BasicPasswordEncryptor passwordEncryptor;

	public ChangeAction() {
		userDAO = new UserDAO();
		passwordEncryptor = new BasicPasswordEncryptor();
	}

	/* (non-Javadoc)
	 * @see actions.Action#getName()
	 */
	@Override
	public String getName() {
		return "change";
	}

	/* (non-Javadoc)
	 * @see actions.Action#perform(javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public String perform(HttpServletRequest request) {
		HttpSession session = request.getSession();
		HibernateUtil.beginTransaction();
	    
		Long userId = (Long)session.getAttribute("userId");
		User user = userDAO.findById(userId, true);
		
		// Set form as a request attribute so we can provide any values back to
		// the user in case of errors
		ChangeForm changeForm = new ChangeForm(request);
        request.setAttribute("form", changeForm);

		// Any validation errors, return to the form and ask again.
		// Note, we set up error list (used to store any errors we might
		// encounter below)
		List<String> errors = changeForm.getErrors();
		request.setAttribute("errors", errors);
		if (!errors.isEmpty()) {
			HibernateUtil.commitTransaction();
			return "change.jsp";
		}

		String encryptedPass = passwordEncryptor.encryptPassword(changeForm.getPass());
		user.setPass(encryptedPass);
		
		userDAO.makePersistent(user);
		HibernateUtil.commitTransaction();
		session.setAttribute("userId", user.getId());
		session.setAttribute("user", user);
		
		return "profile.jsp";
	}

}
