package controller;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import actions.*;
import actions.manage.*;

/**
 * The Class Controller.
 * 
 * Performs the doGet and doPost with some helper methods. All of the doGet
 * requests are forwarded doPost for convenience.
 */
public class Controller extends HttpServlet {

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.GenericServlet#init()
	 */
	@Override
	public void init() throws ServletException {
		Action.add(new LoginAction());
		Action.add(new LogoutAction());
		Action.add(new RegisterAction());
		
		Action.add(new ChangeAction());
		Action.add(new NotesAction());
		Action.add(new ProfileAction());
		Action.add(new SitesAction());
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse)
	 */
	@Override
	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		doPost(request, response);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.http.HttpServlet#doPost(javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse)
	 */
	@Override
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
	
//	    if (true && !request.isSecure()) {
//			// If we're required to use SSL and we're not on a secure connection, redirect
//        	String host = request.getServerName();
//        	String port = (request.getServerPort()==80) ? "" : ":8443";
//            response.sendRedirect("https://"+host+port+request.getRequestURI());
//            return;
//        }
		
		String nextPage = performAction(request);
		sendToNextPage(nextPage, request, response);
	}

	/**
	 * Perform the action. Extracts the requested action and (depending on
	 * whether the user is logged in) perform it (or make the user login).
	 */
	private String performAction(HttpServletRequest request) {
		String servletPath = request.getServletPath();
		HttpSession session = request.getSession();
		
		String action = getActionName(servletPath);
		
		if(action.equals("welcome")) {
			return "index.jsp";
		}
		
		// System.out.println("servletPath="+servletPath+"
		// requestURI="+request.getRequestURI()+" user="+user);		
		Long userId = (Long) session.getAttribute("userId");
		if (userId == null && (action.equals("sites") 
							|| action.equals("note")
							|| action.equals("profile"))) {
			return "index.jsp";
		}

		// Let the logged in user run his chosen action
		return Action.perform(action, request);
	}

	/**
	 * Send to next page. If nextPage is null, send back 404 If nextPage starts
	 * with a '/', redirect to this page. In this case, the page must be the
	 * whole servlet path including the webapp name
	 */
	private void sendToNextPage(String nextPage, HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {

		if (nextPage == null) {
			response.sendError(HttpServletResponse.SC_NOT_FOUND, request.getServletPath());
			return;
		}

		if (nextPage.charAt(0) == '/') {
			String host = request.getServerName();
			String port = ":" + String.valueOf(request.getServerPort());
			if (port.equals(":80"))
				port = "";
			response.sendRedirect("http://" + host + port + nextPage);
			return;
		}

		RequestDispatcher d = request.getRequestDispatcher("/" + nextPage);
		d.forward(request, response);
	}
	
	
	/**
	 * Gets the action name. Returns the path component after the last slash
	 * removing any "extension" if present.
	 */
	private String getActionName(String path) {
		int slash = path.lastIndexOf('/');
		int dot = path.lastIndexOf('.');
		if (dot == -1) dot = path.length();
		return path.substring(slash + 1, dot);
	}

}
