package daos;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.LockMode;
import org.hibernate.Session;
import org.hibernate.criterion.Example;

import util.HibernateUtil;

// TODO: Auto-generated Javadoc
/**
 * The Class GenericDAOHibernate.
 */
public abstract class GenericDAOHibernate<T, ID extends Serializable> {

	protected Log log;
	private Class<T> persistentClass;

	/**
	 * Instantiates a new generic DAO hibernate.
	 */
	@SuppressWarnings("unchecked")
	public GenericDAOHibernate() {
		this.persistentClass = (Class<T>) ((ParameterizedType) getClass()
				.getGenericSuperclass()).getActualTypeArguments()[0];
		// this.sessionFactory = HibernateUtil.getSessionFactory();
		log = LogFactory.getLog(this.getClass());
	}

	/**
	 * Clear.
	 */
	public void clear() {
		getSession().clear();
	}

	/**
	 * Find by example.
	 * 
	 * @param exampleInstance
	 *            the example instance
	 * @param excludeProperty
	 *            the exclude property
	 * 
	 * @return the list< t>
	 */
	@SuppressWarnings("unchecked")
	public List<T> findByExample(T exampleInstance, String[] excludeProperty) {
		Criteria crit = getSession().createCriteria(getPersistentClass());
		Example example = Example.create(exampleInstance);
		for (String exclude : excludeProperty) {
			example.excludeProperty(exclude);
		}
		crit.add(example);
		return crit.list();
	}

	/**
	 * Find by id.
	 * 
	 * @param id
	 *            the id
	 * @param lock
	 *            the lock
	 * 
	 * @return the t
	 */
	@SuppressWarnings("unchecked")
	public T findById(ID id, boolean lock) {
		try {
			T entity;
			if (lock) {
				entity = (T) getSession().get(getPersistentClass(), id,LockMode.UPGRADE);
			} else {
				entity = (T) getSession().get(getPersistentClass(), id);
			}
			return entity;
		} catch (HibernateException e) {
			log.error("unable to find by id=" + id + " entity="+ getPersistentClass());
			return null;
		}
	}

	/**
	 * Flush.
	 */
	public void flush() {
		getSession().flush();
	}

	/**
	 * Gets the persistent class.
	 * 
	 * @return the persistent class
	 */
	protected Class<T> getPersistentClass() {
		return persistentClass;
	}

	/**
	 * Gets the session.
	 * 
	 * @return the session
	 */
	protected Session getSession() {
		return HibernateUtil.getSession();
	}

	/**
	 * Make persistent.
	 * 
	 * @param entity
	 *            the entity
	 * 
	 * @return the t
	 */
	@SuppressWarnings("unchecked")
	public T makePersistent(T entity) {
		getSession().saveOrUpdate(entity);
		return entity;
	}

	/**
	 * Make transient.
	 * 
	 * @param entity
	 *            the entity
	 */
	public void makeTransient(T entity) {
		getSession().delete(entity);
	}

}
